@@ -95,7 +95,7 @@ typedef struct {
 #define NTLM_CHALLENGE_DOMAIN_STRING_OFFSET 12
 
 #define NTLM_RESPONSE_HEADER "NTLMSSP\x00\x03\x00\x00\x00"
-#define NTLM_RESPONSE_FLAGS "\x82\x01"
+#define NTLM_RESPONSE_FLAGS 0x8202
 
 typedef struct {
         guchar     header[12];
@@ -105,27 +105,16 @@ typedef struct {
 	NTLMString domain;
 	NTLMString user;
 	NTLMString host;
+	NTLMString session_key;
 
-        guint16    msg_len;
-        guchar     zero_pad[2];
-
-        guchar     flags[2];
-        guchar     zero_pad2[2];
+        guint32    flags;
 } NTLMResponse;
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-#define LE16(x) (((x & 0xFF) << 8) | ((x >> 8) & 0xFF))
-#define UNLE16(x) LE16(x)
-#else
-#define LE16(x) x
-#define UNLE16(x) x
-#endif
-
 static void
 ntlm_set_string (NTLMString *string, int *offset, int len)
 {
-	string->offset = LE16 (*offset);
-	string->length = string->length2 = LE16 (len);
+	string->offset = GUINT16_TO_LE (*offset);
+	string->length = string->length2 = GUINT16_TO_LE (len);
 	*offset += len;
 }
 
@@ -166,8 +155,8 @@ soup_ntlm_parse_challenge (const char *challenge,
 
 	if (default_domain) {
 		memcpy (&domain, chall + NTLM_CHALLENGE_DOMAIN_STRING_OFFSET, sizeof (domain));
-		domain.length = UNLE16 (domain.length);
-		domain.offset = UNLE16 (domain.offset);
+		domain.length = GUINT16_FROM_LE (domain.length);
+		domain.offset = GUINT16_FROM_LE (domain.offset);
 
 		if (clen < domain.length + domain.offset) {
 			g_free (chall);
@@ -178,8 +167,8 @@ soup_ntlm_parse_challenge (const char *challenge,
 	}
 
 	if (nonce) {
-		*nonce = g_strndup (chall + NTLM_CHALLENGE_NONCE_OFFSET,
-				    NTLM_CHALLENGE_NONCE_LENGTH);
+		*nonce = g_memdup (chall + NTLM_CHALLENGE_NONCE_OFFSET,
+				   NTLM_CHALLENGE_NONCE_LENGTH);
 	}
 
 	return TRUE;
@@ -205,7 +194,7 @@ soup_ntlm_response (const char *nonce,
 
 	memset (&resp, 0, sizeof (resp));
 	memcpy (resp.header, NTLM_RESPONSE_HEADER, sizeof (resp.header));
-	memcpy (resp.flags, NTLM_RESPONSE_FLAGS, sizeof (resp.flags));
+	resp.flags = GUINT32_TO_LE (NTLM_RESPONSE_FLAGS);
 
 	offset = sizeof (resp);
 
@@ -268,9 +257,6 @@ soup_ntlm_response (const char *nonce,
 	return out;
 }
 
-#define KEYBITS(k,s) \
-        (((k[(s)/8] << ((s)%8)) & 0xFF) | (k[(s)/8+1] >> (8-(s)%8)))
-
 /* DES utils */
 /* Set up a key schedule based on a 56bit key */
 static void
@@ -279,15 +265,22 @@ setup_schedule (const guchar *key_56, DES_KS ks)
 	guchar key[8];
 	int i, c, bit;
 
-	for (i = 0; i < 8; i++) {
-		key [i] = KEYBITS (key_56, i * 7);
+	key[0] = (key_56[0])                                 ;
+	key[1] = (key_56[1] >> 1) | ((key_56[0] << 7) & 0xFF);
+	key[2] = (key_56[2] >> 2) | ((key_56[1] << 6) & 0xFF);
+	key[3] = (key_56[3] >> 3) | ((key_56[2] << 5) & 0xFF);
+	key[4] = (key_56[4] >> 4) | ((key_56[3] << 4) & 0xFF);
+	key[5] = (key_56[5] >> 5) | ((key_56[4] << 3) & 0xFF);
+	key[6] = (key_56[6] >> 6) | ((key_56[5] << 2) & 0xFF);
+	key[7] =                    ((key_56[6] << 1) & 0xFF);
 
-		/* Fix parity */
+	/* Fix parity */
+	for (i = 0; i < 8; i++) {
 		for (c = bit = 0; bit < 8; bit++)
-			if (key [i] & (1 << bit))
+			if (key[i] & (1 << bit))
 				c++;
 		if (!(c & 1))
-			key [i] ^= 0x01;
+			key[i] ^= 0x01;
 	}
 
         deskey (ks, key, 0);
@@ -438,134 +431,134 @@ md4sum (const unsigned char *in, int nbytes, unsigned char digest[16])
 
 /* Public domain DES implementation from Phil Karn */
 static unsigned long Spbox[8][64] = {
-	0x01010400,0x00000000,0x00010000,0x01010404,
-	0x01010004,0x00010404,0x00000004,0x00010000,
-	0x00000400,0x01010400,0x01010404,0x00000400,
-	0x01000404,0x01010004,0x01000000,0x00000004,
-	0x00000404,0x01000400,0x01000400,0x00010400,
-	0x00010400,0x01010000,0x01010000,0x01000404,
-	0x00010004,0x01000004,0x01000004,0x00010004,
-	0x00000000,0x00000404,0x00010404,0x01000000,
-	0x00010000,0x01010404,0x00000004,0x01010000,
-	0x01010400,0x01000000,0x01000000,0x00000400,
-	0x01010004,0x00010000,0x00010400,0x01000004,
-	0x00000400,0x00000004,0x01000404,0x00010404,
-	0x01010404,0x00010004,0x01010000,0x01000404,
-	0x01000004,0x00000404,0x00010404,0x01010400,
-	0x00000404,0x01000400,0x01000400,0x00000000,
-	0x00010004,0x00010400,0x00000000,0x01010004,
-	0x80108020,0x80008000,0x00008000,0x00108020,
-	0x00100000,0x00000020,0x80100020,0x80008020,
-	0x80000020,0x80108020,0x80108000,0x80000000,
-	0x80008000,0x00100000,0x00000020,0x80100020,
-	0x00108000,0x00100020,0x80008020,0x00000000,
-	0x80000000,0x00008000,0x00108020,0x80100000,
-	0x00100020,0x80000020,0x00000000,0x00108000,
-	0x00008020,0x80108000,0x80100000,0x00008020,
-	0x00000000,0x00108020,0x80100020,0x00100000,
-	0x80008020,0x80100000,0x80108000,0x00008000,
-	0x80100000,0x80008000,0x00000020,0x80108020,
-	0x00108020,0x00000020,0x00008000,0x80000000,
-	0x00008020,0x80108000,0x00100000,0x80000020,
-	0x00100020,0x80008020,0x80000020,0x00100020,
-	0x00108000,0x00000000,0x80008000,0x00008020,
-	0x80000000,0x80100020,0x80108020,0x00108000,
-	0x00000208,0x08020200,0x00000000,0x08020008,
-	0x08000200,0x00000000,0x00020208,0x08000200,
-	0x00020008,0x08000008,0x08000008,0x00020000,
-	0x08020208,0x00020008,0x08020000,0x00000208,
-	0x08000000,0x00000008,0x08020200,0x00000200,
-	0x00020200,0x08020000,0x08020008,0x00020208,
-	0x08000208,0x00020200,0x00020000,0x08000208,
-	0x00000008,0x08020208,0x00000200,0x08000000,
-	0x08020200,0x08000000,0x00020008,0x00000208,
-	0x00020000,0x08020200,0x08000200,0x00000000,
-	0x00000200,0x00020008,0x08020208,0x08000200,
-	0x08000008,0x00000200,0x00000000,0x08020008,
-	0x08000208,0x00020000,0x08000000,0x08020208,
-	0x00000008,0x00020208,0x00020200,0x08000008,
-	0x08020000,0x08000208,0x00000208,0x08020000,
-	0x00020208,0x00000008,0x08020008,0x00020200,
-	0x00802001,0x00002081,0x00002081,0x00000080,
-	0x00802080,0x00800081,0x00800001,0x00002001,
-	0x00000000,0x00802000,0x00802000,0x00802081,
-	0x00000081,0x00000000,0x00800080,0x00800001,
-	0x00000001,0x00002000,0x00800000,0x00802001,
-	0x00000080,0x00800000,0x00002001,0x00002080,
-	0x00800081,0x00000001,0x00002080,0x00800080,
-	0x00002000,0x00802080,0x00802081,0x00000081,
-	0x00800080,0x00800001,0x00802000,0x00802081,
-	0x00000081,0x00000000,0x00000000,0x00802000,
-	0x00002080,0x00800080,0x00800081,0x00000001,
-	0x00802001,0x00002081,0x00002081,0x00000080,
-	0x00802081,0x00000081,0x00000001,0x00002000,
-	0x00800001,0x00002001,0x00802080,0x00800081,
-	0x00002001,0x00002080,0x00800000,0x00802001,
-	0x00000080,0x00800000,0x00002000,0x00802080,
-	0x00000100,0x02080100,0x02080000,0x42000100,
-	0x00080000,0x00000100,0x40000000,0x02080000,
-	0x40080100,0x00080000,0x02000100,0x40080100,
-	0x42000100,0x42080000,0x00080100,0x40000000,
-	0x02000000,0x40080000,0x40080000,0x00000000,
-	0x40000100,0x42080100,0x42080100,0x02000100,
-	0x42080000,0x40000100,0x00000000,0x42000000,
-	0x02080100,0x02000000,0x42000000,0x00080100,
-	0x00080000,0x42000100,0x00000100,0x02000000,
-	0x40000000,0x02080000,0x42000100,0x40080100,
-	0x02000100,0x40000000,0x42080000,0x02080100,
-	0x40080100,0x00000100,0x02000000,0x42080000,
-	0x42080100,0x00080100,0x42000000,0x42080100,
-	0x02080000,0x00000000,0x40080000,0x42000000,
-	0x00080100,0x02000100,0x40000100,0x00080000,
-	0x00000000,0x40080000,0x02080100,0x40000100,
-	0x20000010,0x20400000,0x00004000,0x20404010,
-	0x20400000,0x00000010,0x20404010,0x00400000,
-	0x20004000,0x00404010,0x00400000,0x20000010,
-	0x00400010,0x20004000,0x20000000,0x00004010,
-	0x00000000,0x00400010,0x20004010,0x00004000,
-	0x00404000,0x20004010,0x00000010,0x20400010,
-	0x20400010,0x00000000,0x00404010,0x20404000,
-	0x00004010,0x00404000,0x20404000,0x20000000,
-	0x20004000,0x00000010,0x20400010,0x00404000,
-	0x20404010,0x00400000,0x00004010,0x20000010,
-	0x00400000,0x20004000,0x20000000,0x00004010,
-	0x20000010,0x20404010,0x00404000,0x20400000,
-	0x00404010,0x20404000,0x00000000,0x20400010,
-	0x00000010,0x00004000,0x20400000,0x00404010,
-	0x00004000,0x00400010,0x20004010,0x00000000,
-	0x20404000,0x20000000,0x00400010,0x20004010,
-	0x00200000,0x04200002,0x04000802,0x00000000,
-	0x00000800,0x04000802,0x00200802,0x04200800,
-	0x04200802,0x00200000,0x00000000,0x04000002,
-	0x00000002,0x04000000,0x04200002,0x00000802,
-	0x04000800,0x00200802,0x00200002,0x04000800,
-	0x04000002,0x04200000,0x04200800,0x00200002,
-	0x04200000,0x00000800,0x00000802,0x04200802,
-	0x00200800,0x00000002,0x04000000,0x00200800,
-	0x04000000,0x00200800,0x00200000,0x04000802,
-	0x04000802,0x04200002,0x04200002,0x00000002,
-	0x00200002,0x04000000,0x04000800,0x00200000,
-	0x04200800,0x00000802,0x00200802,0x04200800,
-	0x00000802,0x04000002,0x04200802,0x04200000,
-	0x00200800,0x00000000,0x00000002,0x04200802,
-	0x00000000,0x00200802,0x04200000,0x00000800,
-	0x04000002,0x04000800,0x00000800,0x00200002,
-	0x10001040,0x00001000,0x00040000,0x10041040,
-	0x10000000,0x10001040,0x00000040,0x10000000,
-	0x00040040,0x10040000,0x10041040,0x00041000,
-	0x10041000,0x00041040,0x00001000,0x00000040,
-	0x10040000,0x10000040,0x10001000,0x00001040,
-	0x00041000,0x00040040,0x10040040,0x10041000,
-	0x00001040,0x00000000,0x00000000,0x10040040,
-	0x10000040,0x10001000,0x00041040,0x00040000,
-	0x00041040,0x00040000,0x10041000,0x00001000,
-	0x00000040,0x10040040,0x00001000,0x00041040,
-	0x10001000,0x00000040,0x10000040,0x10040000,
-	0x10040040,0x10000000,0x00040000,0x10001040,
-	0x00000000,0x10041040,0x00040040,0x10000040,
-	0x10040000,0x10001000,0x10001040,0x00000000,
-	0x10041040,0x00041000,0x00041000,0x00001040,
-	0x00001040,0x00040040,0x10000000,0x10041000,
+	{ 0x01010400,0x00000000,0x00010000,0x01010404,
+	  0x01010004,0x00010404,0x00000004,0x00010000,
+	  0x00000400,0x01010400,0x01010404,0x00000400,
+	  0x01000404,0x01010004,0x01000000,0x00000004,
+	  0x00000404,0x01000400,0x01000400,0x00010400,
+	  0x00010400,0x01010000,0x01010000,0x01000404,
+	  0x00010004,0x01000004,0x01000004,0x00010004,
+	  0x00000000,0x00000404,0x00010404,0x01000000,
+	  0x00010000,0x01010404,0x00000004,0x01010000,
+	  0x01010400,0x01000000,0x01000000,0x00000400,
+	  0x01010004,0x00010000,0x00010400,0x01000004,
+	  0x00000400,0x00000004,0x01000404,0x00010404,
+	  0x01010404,0x00010004,0x01010000,0x01000404,
+	  0x01000004,0x00000404,0x00010404,0x01010400,
+	  0x00000404,0x01000400,0x01000400,0x00000000,
+	  0x00010004,0x00010400,0x00000000,0x01010004 },
+	{ 0x80108020,0x80008000,0x00008000,0x00108020,
+	  0x00100000,0x00000020,0x80100020,0x80008020,
+	  0x80000020,0x80108020,0x80108000,0x80000000,
+	  0x80008000,0x00100000,0x00000020,0x80100020,
+	  0x00108000,0x00100020,0x80008020,0x00000000,
+	  0x80000000,0x00008000,0x00108020,0x80100000,
+	  0x00100020,0x80000020,0x00000000,0x00108000,
+	  0x00008020,0x80108000,0x80100000,0x00008020,
+	  0x00000000,0x00108020,0x80100020,0x00100000,
+	  0x80008020,0x80100000,0x80108000,0x00008000,
+	  0x80100000,0x80008000,0x00000020,0x80108020,
+	  0x00108020,0x00000020,0x00008000,0x80000000,
+	  0x00008020,0x80108000,0x00100000,0x80000020,
+	  0x00100020,0x80008020,0x80000020,0x00100020,
+	  0x00108000,0x00000000,0x80008000,0x00008020,
+	  0x80000000,0x80100020,0x80108020,0x00108000 },
+	{ 0x00000208,0x08020200,0x00000000,0x08020008,
+	  0x08000200,0x00000000,0x00020208,0x08000200,
+	  0x00020008,0x08000008,0x08000008,0x00020000,
+	  0x08020208,0x00020008,0x08020000,0x00000208,
+	  0x08000000,0x00000008,0x08020200,0x00000200,
+	  0x00020200,0x08020000,0x08020008,0x00020208,
+	  0x08000208,0x00020200,0x00020000,0x08000208,
+	  0x00000008,0x08020208,0x00000200,0x08000000,
+	  0x08020200,0x08000000,0x00020008,0x00000208,
+	  0x00020000,0x08020200,0x08000200,0x00000000,
+	  0x00000200,0x00020008,0x08020208,0x08000200,
+	  0x08000008,0x00000200,0x00000000,0x08020008,
+	  0x08000208,0x00020000,0x08000000,0x08020208,
+	  0x00000008,0x00020208,0x00020200,0x08000008,
+	  0x08020000,0x08000208,0x00000208,0x08020000,
+	  0x00020208,0x00000008,0x08020008,0x00020200 },
+	{ 0x00802001,0x00002081,0x00002081,0x00000080,
+	  0x00802080,0x00800081,0x00800001,0x00002001,
+	  0x00000000,0x00802000,0x00802000,0x00802081,
+	  0x00000081,0x00000000,0x00800080,0x00800001,
+	  0x00000001,0x00002000,0x00800000,0x00802001,
+	  0x00000080,0x00800000,0x00002001,0x00002080,
+	  0x00800081,0x00000001,0x00002080,0x00800080,
+	  0x00002000,0x00802080,0x00802081,0x00000081,
+	  0x00800080,0x00800001,0x00802000,0x00802081,
+	  0x00000081,0x00000000,0x00000000,0x00802000,
+	  0x00002080,0x00800080,0x00800081,0x00000001,
+	  0x00802001,0x00002081,0x00002081,0x00000080,
+	  0x00802081,0x00000081,0x00000001,0x00002000,
+	  0x00800001,0x00002001,0x00802080,0x00800081,
+	  0x00002001,0x00002080,0x00800000,0x00802001,
+	  0x00000080,0x00800000,0x00002000,0x00802080 },
+	{ 0x00000100,0x02080100,0x02080000,0x42000100,
+	  0x00080000,0x00000100,0x40000000,0x02080000,
+	  0x40080100,0x00080000,0x02000100,0x40080100,
+	  0x42000100,0x42080000,0x00080100,0x40000000,
+	  0x02000000,0x40080000,0x40080000,0x00000000,
+	  0x40000100,0x42080100,0x42080100,0x02000100,
+	  0x42080000,0x40000100,0x00000000,0x42000000,
+	  0x02080100,0x02000000,0x42000000,0x00080100,
+	  0x00080000,0x42000100,0x00000100,0x02000000,
+	  0x40000000,0x02080000,0x42000100,0x40080100,
+	  0x02000100,0x40000000,0x42080000,0x02080100,
+	  0x40080100,0x00000100,0x02000000,0x42080000,
+	  0x42080100,0x00080100,0x42000000,0x42080100,
+	  0x02080000,0x00000000,0x40080000,0x42000000,
+	  0x00080100,0x02000100,0x40000100,0x00080000,
+	  0x00000000,0x40080000,0x02080100,0x40000100 },
+	{ 0x20000010,0x20400000,0x00004000,0x20404010,
+	  0x20400000,0x00000010,0x20404010,0x00400000,
+	  0x20004000,0x00404010,0x00400000,0x20000010,
+	  0x00400010,0x20004000,0x20000000,0x00004010,
+	  0x00000000,0x00400010,0x20004010,0x00004000,
+	  0x00404000,0x20004010,0x00000010,0x20400010,
+	  0x20400010,0x00000000,0x00404010,0x20404000,
+	  0x00004010,0x00404000,0x20404000,0x20000000,
+	  0x20004000,0x00000010,0x20400010,0x00404000,
+	  0x20404010,0x00400000,0x00004010,0x20000010,
+	  0x00400000,0x20004000,0x20000000,0x00004010,
+	  0x20000010,0x20404010,0x00404000,0x20400000,
+	  0x00404010,0x20404000,0x00000000,0x20400010,
+	  0x00000010,0x00004000,0x20400000,0x00404010,
+	  0x00004000,0x00400010,0x20004010,0x00000000,
+	  0x20404000,0x20000000,0x00400010,0x20004010 },
+	{ 0x00200000,0x04200002,0x04000802,0x00000000,
+	  0x00000800,0x04000802,0x00200802,0x04200800,
+	  0x04200802,0x00200000,0x00000000,0x04000002,
+	  0x00000002,0x04000000,0x04200002,0x00000802,
+	  0x04000800,0x00200802,0x00200002,0x04000800,
+	  0x04000002,0x04200000,0x04200800,0x00200002,
+	  0x04200000,0x00000800,0x00000802,0x04200802,
+	  0x00200800,0x00000002,0x04000000,0x00200800,
+	  0x04000000,0x00200800,0x00200000,0x04000802,
+	  0x04000802,0x04200002,0x04200002,0x00000002,
+	  0x00200002,0x04000000,0x04000800,0x00200000,
+	  0x04200800,0x00000802,0x00200802,0x04200800,
+	  0x00000802,0x04000002,0x04200802,0x04200000,
+	  0x00200800,0x00000000,0x00000002,0x04200802,
+	  0x00000000,0x00200802,0x04200000,0x00000800,
+	  0x04000002,0x04000800,0x00000800,0x00200002 },
+	{ 0x10001040,0x00001000,0x00040000,0x10041040,
+	  0x10000000,0x10001040,0x00000040,0x10000000,
+	  0x00040040,0x10040000,0x10041040,0x00041000,
+	  0x10041000,0x00041040,0x00001000,0x00000040,
+	  0x10040000,0x10000040,0x10001000,0x00001040,
+	  0x00041000,0x00040040,0x10040040,0x10041000,
+	  0x00001040,0x00000000,0x00000000,0x10040040,
+	  0x10000040,0x10001000,0x00041040,0x00040000,
+	  0x00041040,0x00040000,0x10041000,0x00001000,
+	  0x00000040,0x10040040,0x00001000,0x00041040,
+	  0x10001000,0x00000040,0x10000040,0x10040000,
+	  0x10040040,0x10000000,0x00040000,0x10001040,
+	  0x00000000,0x10041040,0x00040040,0x10000040,
+	  0x10040000,0x10001000,0x10001040,0x00000000,
+	  0x10041040,0x00041000,0x00041000,0x00001040,
+	  0x00001040,0x00040040,0x10000000,0x10041000 }
 };
 
 #undef F
